#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), canvasWidget(new QWidget(this))
{
    QMenu *figureMenu = menuBar()->addMenu("Фігура");
    QMenu *clearMenu = menuBar()->addMenu("Стерти");
    QAction *drawCircleAction = new QAction("Коло", this);
    QAction *clearAllAction = new QAction("Стерти все", this);
    figureMenu->addAction(drawCircleAction);
    clearMenu->addAction(clearAllAction);
    connect(drawCircleAction, &QAction::triggered, this, &MainWindow::showCircleDialog);
    connect(clearAllAction, &QAction::triggered, this, &MainWindow::clearShapes);

    setCentralWidget(canvasWidget);
    resize(800, 600);
}

MainWindow::~MainWindow() {}

//функція запиту даних для малювання фігури
void MainWindow::showCircleDialog()
{
    QDialog *circleDialog = new QDialog(this);
    circleDialog->setWindowTitle("Параметри фігури");
    QVBoxLayout *layout = new QVBoxLayout(circleDialog);

    QSize windowSize = this->size();

    QSpinBox *xInput = new QSpinBox(circleDialog);
    xInput->setRange(-windowSize.width() / 2, windowSize.width() / 2);
    QSpinBox *yInput = new QSpinBox(circleDialog);
    yInput->setRange(-windowSize.height() / 2, windowSize.height() / 2);
    QSpinBox *radiusInput = new QSpinBox(circleDialog);
    radiusInput->setRange(0, 300);
    QPushButton *circleColorButton = new QPushButton("Вибрати колір кола", circleDialog);
    QPushButton *hexagonColorButton = new QPushButton("Вибрати колір шестикутника", circleDialog);
    QPushButton *confirmButton = new QPushButton("Намалювати", circleDialog);

    QCheckBox *hexagonCheckBox = new QCheckBox("Намалювати шестикутник", circleDialog);
    QCheckBox *radiusCheckBox = new QCheckBox("Намалювати радіус", circleDialog);
    QCheckBox *centreCheckBox = new QCheckBox("Підписати центр", circleDialog);
    hexagonColorButton->setEnabled(false);

    layout->addWidget(new QLabel("X:                                                                                            "));
    layout->addWidget(xInput);
    layout->addWidget(new QLabel("Y:"));
    layout->addWidget(yInput);
    layout->addWidget(new QLabel("Радіус:"));
    layout->addWidget(radiusInput);
    layout->addWidget(circleColorButton);
    layout->addWidget(radiusCheckBox);
    layout->addWidget(centreCheckBox);
    layout->addWidget(hexagonCheckBox);
    layout->addWidget(hexagonColorButton);
    layout->addWidget(confirmButton);

    connect(hexagonCheckBox, &QCheckBox::checkStateChanged, [hexagonColorButton](int state) {
        hexagonColorButton->setEnabled(state == Qt::Checked);
    });

    connect(circleColorButton, &QPushButton::clicked, [this]() {
        QColor color = QColorDialog::getColor(Qt::white, this, "Вибір кольору кола");
        if (color.isValid()) {
            this->circleColor = color;
        }
    });

    connect(hexagonColorButton, &QPushButton::clicked, [this]() {
        QColor color = QColorDialog::getColor(Qt::white, this, "Вибір кольору шестикутника");
        if (color.isValid()) {
            this->hexagonColor = color;
        }
    });

    connect(confirmButton, &QPushButton::clicked, [this, xInput, yInput, radiusInput, hexagonCheckBox, radiusCheckBox, centreCheckBox, circleDialog]() {
        this->shapes.append({xInput->value(), yInput->value(), radiusInput->value(), circleColor, hexagonCheckBox->isChecked(), hexagonColor, radiusCheckBox->isChecked(), centreCheckBox->isChecked()});
        circleDialog->close();

        //повідомлення помилки, якщо користувач малює фігуру, жодна точка якої не належить третій чверті координатної площини
        if ((xInput->value() > 0 && xInput->value() - radiusInput->value() >= 0) && (yInput->value() > 0 && yInput->value() - radiusInput->value()>= 0)) {
            QMessageBox::critical(this, "Помилка", "Фігура не знаходиться в третій чверті, її не буде видно.");
            return;
        }

        //повідомлення попередження, якщо користувач малює фігуру, частина якої не належить третій чверті координатної площини
        if (((xInput->value() >= 0 && xInput->value() - radiusInput->value() >= 0) || (xInput->value() <= 0 && xInput->value() + radiusInput->value() >= 0)) || (((yInput->value() > 0) && ((yInput->value() - radiusInput->value()) >= 0)) || (yInput->value() < 0 && yInput->value() + radiusInput->value() >= 0))) {
            QMessageBox::warning(this, "Попередження", "Частина фігури виходить за межі третьої чверті, її не буде видно повністю.");
        }

        update();
    });

    circleDialog->setLayout(layout);
    circleDialog->exec();
}

//функція малювання фігур та координатної площини
void MainWindow::paintEvent(QPaintEvent*)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QRect canvasRect = canvasWidget->geometry();
    int originX = canvasRect.width() / 2;
    int originY = canvasRect.height() / 2;

    //малювання фігур
    for (const auto &shape : shapes) {
        int xreal = originX + shape.x;
        int yreal = originY - shape.y;
        painter.setBrush(shape.circleColor);
        painter.drawEllipse(xreal - shape.r, yreal - shape.r, shape.r * 2, shape.r * 2);

        if (shape.drawHexagon) {
            QPointF hexagon[6];
            for (int i = 0; i < 6; ++i) {
                float angle = M_PI / 3 * i;
                hexagon[i] = QPointF(xreal + shape.r * cos(angle), yreal + shape.r * sin(angle));
            }
            painter.setBrush(shape.hexagonColor);
            painter.drawPolygon(hexagon, 6);
        }

        //малювання радіуса кола
        if (shape.drawRadius) {
            painter.setPen(Qt::black);
            painter.drawLine(xreal, yreal, xreal + shape.r * cos(M_PI * 1.25), yreal + shape.r * sin(- M_PI * 1.25));
        }

        //малювання центру кола
        if (shape.showCentre) {
            painter.setPen(Qt::black);
            painter.drawText(xreal - 5, yreal - 10, "O");
        }
    }

    //стирання всього окрім 3 чверті координатної площини
    painter.eraseRect(canvasRect.width() / 2, 0, canvasRect.width() / 2, canvasRect.height() / 2);
    painter.eraseRect(0, 0, canvasRect.width() / 2, canvasRect.height() / 2);
    painter.eraseRect(canvasRect.width() / 2, canvasRect.height() / 2, canvasRect.width() / 2, canvasRect.height() / 2);

    //малювання координатної сітки
    painter.setPen(Qt::white);
    //painter.drawLine(0, originY, canvasRect.width(), originY); // X
    //painter.drawLine(originX, 30, originX, canvasRect.height()); // Y

    //одиничні відрізки
    for (int i = originX; i < canvasRect.width()-15; i += 50) {
        painter.drawLine(i, originY - 5, i, originY + 5);
    }
    for (int i = originY; i < canvasRect.height(); i += 50) {
        painter.drawLine(originX - 5, i, originX + 5, i);
    }

    for (int i = originX; i > 0; i -= 50) {
        painter.drawLine(i, originY - 5, i, originY + 5);
    }
    for (int i = originY; i > 45; i -= 50) {
        painter.drawLine(originX - 5, i, originX + 5, i);
    }

    //довжини одиничних відрізків та початок координат
    painter.drawText(originX + 5, originY + 15, "0");
    painter.drawText(originX + 42, originY - 15, "50");
    painter.drawText(originX - 60, originY + 20, "-50");
    painter.drawText(originX + 10, originY - 45, "50");
    painter.drawText(originX - 28, originY + 54, "-50");

    //стрілки (напрямок координатних прямих)
    painter.drawLine(canvasRect.width() - 10, originY - 5, canvasRect.width(), originY);
    painter.drawLine(canvasRect.width() - 10, originY + 5, canvasRect.width(), originY);
    painter.drawLine(originX - 5, 40, originX, 30);
    painter.drawLine(originX + 5, 40, originX, 30);

    //назви координатних осей
    painter.drawText(canvasRect.width() - 15, originY - 10, "X");
    painter.drawText(originX + 10, 45, "Y");
}

//функція стерти все
void MainWindow::clearShapes() {
    shapes.clear();
    update();
}
